%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% SCRIPT TO ANALYZE THE ACCURACY OF LOSS MEASUREMENT %%%%%%%%%%%%%%%%%
%%%% Prerequesite:
%%%%   - Test the trace for stationarity using stationarityCheck.m
%%%%   - and renewal properties using renewal.m
%%%%
%%%% INPUT: (1) probes sending times, stored in a file named Txyz.txt
%%%%        (2) probe outcomes, stored in a file name xyz.txt
%%%%        wher xyz is the trace number
%%%%         NOTE: 1 means good, 0 means loss
%%%%        (3) the renewalTrace.txt file to indicate whether the trace is
%%%%         renewal
%%%% OUTPUT:(1) The loss rate
%%%%        (2) The variance
%%%%        (3) The 95% confidence interval
%%%%        (4) The mean ON/OFF durations
%%%% COPYRIGHT : Hung X. Nguyen, the University of Adelaide, Australia
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
disp('Preparing the paths for function calls...');
addpath LIBRARY


%%%% important parameters
thinningS=2; % subdivide the trace into equal segment
predict=0; % perform prediction (predict=1) or cross validation (predict=0)
samplingUnit=1;%10^9; %%% Nanosecond

%%%% Path to the data directory
dir =['../DATA/SAIL/'];
nrTraces =1781; %%% FIXME: Should get this number from the given directory automtically

kArray=[];
thetaArray=[];
renewalArray =[];
stationaryArray=[];
LRArray=[];
LRINFER=[];
LREMP=[];
LRVAL=[];
VARINFER=[];
VAREMP=[];
VARVAL=[];
CONF=[];

renewalTrce= [49    50    51    53    70    86   135   138   141   145   146   154   155   162   163   170  171   175   178   179   181   182];
%%%%%%%%%%% starting the SAIL algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%
%for trnum=1:nrTraces
for tid=1:length(renewalTrce)
    
    trnum = renewalTrce(tid);
    
    disp(trnum)   
    tmpTrce=[];
    trce=[];
    
    filename = [dir,int2str(trnum),'.mat'];
        
    if exist(filename)
        
        load(filename);
        trce=1-tmpTrce;
        
        LRArray=[LRArray mean(trce)];
               
        filename = [dir,'T',int2str(trnum),'.mat'];
        
        if exist(filename)
            
            disp('Preparing the data ...');
            
            load(filename); %%% get tmpTrceT
            trceTime=tmpTrceT/samplingUnit; %%% Get time to the unit of seconds
            
            %%%% get the sampling rate
            diff=[];
            for i=1:min(2000,length(trceTime))-1
                diff=[diff (trceTime(i+1)-trceTime(i))];
            end
            
            %%%mean(diff)
            
            si = mean(diff)/samplingUnit;
            lambda=1/si;
            
            clear diff;
            
            
            inferTrce =[];
            validTrce =[];
            
            
            %%%%% CROSS VALIDATION or Prediction
            if (predict == 0)
                randArray=rand(1,length(trce));
                tempk1=find(randArray<=1/thinningS);
                tempk2=find(randArray>1/thinningS);
            else
                %%%% PREDICTION !!!!!
                tempk1=[1:ceil(length(trce)/thinningS)];
                tempk2=[(ceil(length(trce)/thinningS)+1):1:length(trce)];
            end
            
            lambda =lambda/thinningS;
            
            
            inferTrce= trce(tempk1);
            validTrce = trce(tempk2);
            trceTime=trceTime(tempk1);
            
            %trceTime=sort(trceTime); %%% PlanetLab sometime write
            %packets out of order
            
            trce= inferTrce;
            lag = -100;
            
            %Checking for stationarity
            % SImulated traces should pass this test
            stationary=isStationary(trce);
            renewal=isRenewal(trce);
            renewalArray=[renewalArray renewal];           
            if (renewal ==1)
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % Calling SAIL
                disp('Calling SAIL ...');
                tic
                [k theta VEST]=SAIL(inferTrce,trceTime);
                toc
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %Compute confidence intervals
                disp('Computing the confidence interval ..');
                intType =1 ; %1 is Guassian, 2 is Wilson, 3 is Jeffreys
                [upper lower]=getConfidenceInterval(inferTrce,VEST,intType);
                
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%% Collecting statistics
                disp('Collecting statistics ...');
                kArray=[kArray;k];
                thetaArray=[thetaArray;theta];
                stationaryArray=[stationaryArray stationary];
                renewalArray=[renewalArray renewal];
                
                LRVAL=[LRVAL mean(1-validTrce)];
                LRINFER=[LRINFER theta(1)*k(1)/sum(theta.*k)];
                LREMP=[LREMP mean(1-inferTrce)];
                
                CONF=[CONF; [upper lower]];
                
                VARINFER=[VARINFER VEST];
                
                c=autocov(inferTrce,0:1:length(inferTrce)-2);
                c=c(find(~isnan(c)));
                VAREMP=[VAREMP (var(inferTrce) + 2*sum(c(2:length(c))'.*(1-[1:(length(c)-1)]/length(inferTrce))))/length(inferTrce)];
                
                c=autocov(validTrce,0:1:length(validTrce)-2);
                c=c(find(~isnan(c)));
                VARVAL=[VARVAL (var(validTrce) +2*sum(c(2:length(c))'.*(1-[1:(length(c)-1)]/length(validTrce))))/length(validTrce)];
                
            end %%%%isrenewal
        end %%% exist trace file
    end %%% exist time trace   
end %for trnum

